@@ -9,8 +9,8 @@
 #
 package Test::Moose::More;
 our $AUTHORITY = 'cpan:RSRCHBOY';
-# git description: 0.040-16-gb23f2af
-$Test::Moose::More::VERSION = '0.041'; # TRIAL
+# git description: 0.043-29-g7f7738f
+$Test::Moose::More::VERSION = '0.044'; # TRIAL
 
 # ABSTRACT: More tools for testing Moose packages
 
@@ -26,6 +26,7 @@ use Sub::Exporter::Progressive -setup => {
         does_ok
         has_attribute_ok
         has_method_ok
+        has_no_method_ok
         is_anon_ok
         is_class_ok
         is_immutable_ok
@@ -50,6 +51,7 @@ use Sub::Exporter::Progressive -setup => {
         is_role
 
         does_metaroles_ok
+        does_not_metaroles_ok
     } ],
     groups => {
         default  => [ ':all' ],
@@ -64,6 +66,7 @@ use Scalar::Util 'blessed';
 use Syntax::Keyword::Junction 'any';
 use Moose::Util 'resolve_metatrait_alias', 'does_role', 'find_meta';
 use Moose::Util::TypeConstraints;
+use Carp 'confess';
 use Data::OptList;
 
 use Test::Moose::More::Utils;
@@ -166,15 +169,26 @@ sub has_attribute_ok ($$;$) {
 }
 
 
-sub has_method_ok ($@) {
-    my ($thing, @methods) = @_;
+my $_has_test = sub { $tb->ok(!!$_[0]->has_method($_), "$_[1] has method $_")           };
+my $_no_test  = sub { $tb->ok( !$_[0]->has_method($_), "$_[1] does not have method $_") };
+
+sub has_no_method_ok ($@) { unshift @_, $_no_test;  goto \&_method_ok_guts }
+sub has_method_ok    ($@) { unshift @_, $_has_test; goto \&_method_ok_guts }
+
+sub _method_ok_guts {
+    my ($_test, $thing, @methods) = @_;
 
     ### $thing
     my $meta = find_meta($thing);
     my $name = _thing_name($thing, $meta);
 
+    # the test below is run one stack frame up (down?), so let's handle that
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    # "tiny evil?" -- Eleanor Weyl
+
     ### @methods
-    $tb->ok(!!$meta->has_method($_), "$name has method $_")
+    $_test->($meta => $name)
         for @methods;
 
     return;
@@ -313,8 +327,11 @@ sub check_sugar_ok ($) {
 }
 
 
-sub does_metaroles_ok($$) {
-    my ($thing, $metaroles) = @_;
+sub does_metaroles_ok($$)     { push @_, \&does_ok;     goto &_does_metaroles }
+sub does_not_metaroles_ok($$) { push @_, \&does_not_ok; goto &_does_metaroles }
+
+sub _does_metaroles {
+    my ($thing, $metaroles, $test_func) = @_;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
 
@@ -326,7 +343,7 @@ sub does_metaroles_ok($$) {
         my $mop_metaclass = get_mop_metaclass_for $mop => $meta;
 
         local $THING_NAME = "${name}'s $mop metaclass $mop_metaclass";
-        does_ok $mop_metaclass => $metaroles->{$mop};
+        $test_func->($mop_metaclass => $metaroles->{$mop});
     }
 
     return;
@@ -356,6 +373,9 @@ sub _validate_thing_guts {
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
 
+    my $meta = find_meta($thing);
+    my $name = _thing_name($thing, $meta);
+
     ### anonymous...
     $args{anonymous} ? is_anon_ok $thing : is_not_anon_ok $thing
         if exists $args{anonymous};
@@ -364,6 +384,18 @@ sub _validate_thing_guts {
     $args{sugar} ? check_sugar_ok $thing : check_sugar_removed_ok $thing
         if exists $args{sugar};
 
+    # metaclass checking
+    for my $mop (sort keys %{ $args{metaclasses} || {} }) {
+
+        my $mop_metaclass = get_mop_metaclass_for $mop => $meta;
+
+        local $THING_NAME = "${name}'s $mop metaclass";
+        validate_class $mop_metaclass => (
+            -subtest => "Checking the $mop metaclass, $mop_metaclass",
+            %{ $args{metaclasses}->{$mop} },
+        );
+    }
+
     ### roles...
     do { does_ok($thing, $_) for @{$args{does}} }
         if exists $args{does};
@@ -373,6 +405,8 @@ sub _validate_thing_guts {
     ### methods...
     do { has_method_ok($thing, $_) for @{$args{methods}} }
         if exists $args{methods};
+    do { has_no_method_ok($thing, $_) for @{$args{no_methods}} }
+        if exists $args{no_methods};
 
     ### attributes...
     ATTRIBUTE_LOOP:
@@ -405,7 +439,9 @@ sub _validate_class_guts {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unless is_class_ok $class;
 
-    my $name = _thing_name($class);
+    my $meta = find_meta($class);
+    my $name = _thing_name($class, $meta);
+
     do { ok($class->isa($_), "$name isa $_") for @{$args{isa}} }
         if exists $args{isa};
 
@@ -415,6 +451,18 @@ sub _validate_class_guts {
     do { is_not_immutable_ok $class }
         if exists $args{immutable} && !$args{immutable};
 
+    # metaclass / metarole checking
+    do { does_metaroles_ok $class => $args{class_metaroles} }
+        if exists $args{class_metaroles};
+    do { does_not_metaroles_ok $class => $args{no_class_metaroles} }
+        if exists $args{no_class_metaroles};
+
+    confess 'Cannot specify both a metaclasses and class_metaclasses to validate_class()!'
+        if $args{class_metaclasses} && $args{metaclasses};
+
+    $args{metaclasses} = $args{class_metaclasses}
+        if exists $args{class_metaclasses};
+
     return validate_thing $class => %args;
 }
 
@@ -440,6 +488,20 @@ sub _validate_role_guts {
     role_wraps_after_method_ok($role => @{ $args{after} })
         if defined $args{after};
 
+    # metarole checking
+    do { does_metaroles_ok $role => $args{role_metaroles} }
+        if exists $args{role_metaroles};
+    do { does_not_metaroles_ok $role => $args{no_role_metaroles} }
+        if exists $args{no_role_metaroles};
+
+
+    confess 'Cannot specify both a metaclasses and role_metaclasses to validate_class()!'
+        if $args{role_metaclasses} && $args{metaclasses};
+
+    $args{metaclasses} = $args{role_metaclasses}
+        if exists $args{role_metaclasses};
+
+    # if we've been asked to compose ourselves, then do that -- otherwise return
     $args{-compose}
         ?        validate_thing $role => %args
         : return validate_thing $role => %args
@@ -645,9 +707,7 @@ __END__
 
 =encoding UTF-8
 
-=for :stopwords Chris Weyl Chad Etheridge Granum Karen subtest MOPs
-
-=for :stopwords Wishlist flattr flattr'ed gittip gittip'ed
+=for :stopwords Chris Weyl Chad Etheridge Granum Karen subtest MOPs metaroles
 
 =head1 NAME
 
@@ -655,7 +715,7 @@ Test::Moose::More - More tools for testing Moose packages
 
 =head1 VERSION
 
-This document describes version 0.041 of Test::Moose::More - released November 12, 2016 as part of Test-Moose-More.
+This document describes version 0.044 of Test::Moose::More - released March 21, 2017 as part of Test-Moose-More.
 
 =head1 SYNOPSIS
 
@@ -691,6 +751,13 @@ L</validate_attribute>, L</validate_class>, L</validate_role>, L</validate_thing
 
 =back
 
+=head1 FUNCTIONS
+
+=head2 does_not_metaroles_ok $thing => { $mop => [ @traits ], ... };
+
+As with L</does_metaroles_ok>, but test that the metaroles are not consumed, a
+la L</does_not_ok>.
+
 =head1 TEST FUNCTIONS
 
 =head2 meta_ok $thing
@@ -723,6 +790,17 @@ class name, instance, or role name.
 
 Queries $thing's metaclass to see if $thing has the methods named in @methods.
 
+Note: This does B<not> include inherited methods; see
+L<Class::MOP::Class/has_method>.
+
+=head2 has_no_method_ok $thing, @methods
+
+Queries $thing's metaclass to ensure $thing does not provide the methods named
+in @methods.
+
+Note: This does B<not> include inherited methods; see
+L<Class::MOP::Class/has_method>.
+
 =head2 role_wraps_around_method_ok $role, @methods
 
 Queries $role's metaclass to see if $role wraps the methods named in
@@ -842,6 +920,13 @@ The MOPs available for roles (L<Moose::Meta::Role>) are:
 
 =back
 
+Note!  Neither this function nor does_not_metaroles_ok() attempts to validate
+that the MOP type passed in is a member of the above lists.  There's no gain
+here in implementing such a check, and a negative to be had: specifying an
+invalid MOP type will result in immediate explosions, while it's entirely
+possible other MOP types will be added (either to core, via traits, or "let's
+subclass Moose::Meta::Class/etc and implement something new").
+
 =head2 validate_thing
 
 Runs a bunch of tests against the given C<$thing>, as defined:
@@ -920,7 +1005,13 @@ same arguments L</validate_attribute> does.  e.g.:
 
 methods => [ ... ]
 
-A list of methods the thing should have.
+A list of methods the thing should have; see L</has_method_ok>.
+
+=item *
+
+no_methods => [ ... ]
+
+A list of methods the thing should not have; see L</has_no_method_ok>.
 
 =item *
 
@@ -928,6 +1019,39 @@ sugar => 0|1
 
 Ensure that thing can/cannot do the standard Moose sugar.
 
+=item *
+
+metaclasses => { $mop => { ... }, ... }
+
+Validates this thing's metaclasses: that is, given a MOP type (e.g. class,
+attribute, method, ...) and a hashref, find the associated metaclass of the
+given type and invoke L</validate_thing> on it, using the hashref as options
+for validate_thing().
+
+e.g.
+
+    validate_thing 'TestClass' => (
+        metaclasses  => {
+            attribute => {
+                isa  => [ 'Moose::Meta::Attribute' ],
+                does => [ 'MetaRole::attribute'    ],
+            },
+        },
+    );
+
+...yields:
+
+    # Subtest: Checking the attribute metaclass, Moose::Meta::Class::__ANON__::SERIAL::1
+        ok 1 - TestClass's attribute metaclass has a metaclass
+        ok 2 - TestClass's attribute metaclass is a Moose class
+        ok 3 - TestClass's attribute metaclass isa Moose::Meta::Attribute
+        ok 4 - TestClass's attribute metaclass does MetaRole::attribute
+        1..4
+    ok 1 - Checking the attribute metaclass, Moose::Meta::Class::__ANON__::SERIAL::1
+
+Note that validate_class() and validate_role() implement this using
+'class_metaclasses' and 'role_metaclasses', respectively.
+
 =back
 
 =head2 validate_role
@@ -1012,6 +1136,60 @@ A list of methods the role expects to wrap after, on application to a class.
 
 See L<Moose/after> for information on after method modifiers.
 
+=item *
+
+role_metaroles => { $mop => [ $role, ... ], ... }
+
+Checks metaclasses to ensure the given metaroles are applied.  See
+L</does_metaroles_ok>.
+
+=item *
+
+no_role_metaroles => { $mop => [ $role, ... ], ... }
+
+Checks metaclasses to ensure the given metaroles are applied.  See
+L</does_not_metaroles_ok>.
+
+=item *
+
+role_metaclasses => { $mop => { ... }, ... }
+
+Validates this role's metaclasses: that is, given a MOP type (e.g. role,
+attribute, method, ...) and a hashref, find the associated metaclass of the
+given type and invoke L</validate_thing> on it, using the hashref as options
+for validate_thing().
+
+e.g.
+
+    validate_role 'TestRole' => (
+        metaclasses  => {
+            attribute => {
+                isa  => [ 'Moose::Meta::Attribute' ],
+                does => [ 'MetaRole::attribute'    ],
+            },
+        },
+    );
+
+...yields:
+
+    # Subtest: Checking the attribute metaclass, Moose::Meta::Class::__ANON__::SERIAL::1
+        ok 1 - TestRole's attribute metaclass has a metaclass
+        ok 2 - TestRole's attribute metaclass is a Moose class
+        ok 3 - TestRole's attribute metaclass isa Moose::Meta::Attribute
+        ok 4 - TestRole's attribute metaclass does MetaRole::attribute
+        1..4
+    ok 1 - Checking the attribute metaclass, Moose::Meta::Class::__ANON__::SERIAL::1
+
+Note that validate_class() and validate_role() implement this using
+'class_metaclasses' and 'role_metaclasses', respectively.
+
+=item *
+
+class_metaclasses => { $mop => { ... }, ... }
+
+As with role_metaclasses, above, except that this option is only used
+if -compose is also specified.
+
 =back
 
 =head2 validate_class
@@ -1053,6 +1231,52 @@ immutable => 0|1
 
 Checks the class to see if it is/isn't immutable.
 
+=item *
+
+class_metaroles => { $mop => [ $role, ... ], ... }
+
+Checks metaclasses to ensure the given metaroles are applied.  See
+L</does_metaroles_ok>.
+
+=item *
+
+no_class_metaroles => { $mop => [ $role, ... ], ... }
+
+Checks metaclasses to ensure the given metaroles are applied.  See
+L</does_not_metaroles_ok>.
+
+=item *
+
+class_metaclasses => { $mop => { ... }, ... }
+
+Validates this class' metaclasses: that is, given a MOP type (e.g. role,
+attribute, method, ...) and a hashref, find the associated metaclass of the
+given type and invoke L</validate_thing> on it, using the hashref as options
+for validate_thing().
+
+e.g.
+
+    validate_class 'TestClass' => (
+        metaclasses  => {
+            attribute => {
+                isa  => [ 'Moose::Meta::Attribute' ],
+                does => [ 'MetaRole::attribute'    ],
+            },
+        },
+    );
+
+...yields:
+
+    ok 1 - TestClass has a metaclass
+    ok 2 - TestClass is a Moose class
+    # Subtest: Checking the attribute metaclass, Moose::Meta::Class::__ANON__::SERIAL::1
+        ok 1 - TestClass's attribute metaclass has a metaclass
+        ok 2 - TestClass's attribute metaclass is a Moose class
+        ok 3 - TestClass's attribute metaclass isa Moose::Meta::Attribute
+        ok 4 - TestClass's attribute metaclass does MetaRole::attribute
+        1..4
+    ok 3 - Checking the attribute metaclass, Moose::Meta::Class::__ANON__::SERIAL::1
+
 =back
 
 =head2 validate_attribute
@@ -1205,25 +1429,6 @@ feature.
 
 Chris Weyl <cweyl@alumni.drew.edu>
 
-=head2 I'm a material boy in a material world
-
-=begin html
-
-<a href="https://gratipay.com/RsrchBoy/"><img src="http://img.shields.io/gratipay/RsrchBoy.svg" /></a>
-<a href="http://bit.ly/rsrchboys-wishlist"><img src="http://wps.io/wp-content/uploads/2014/05/amazon_wishlist.resized.png" /></a>
-<a href="https://flattr.com/submit/auto?user_id=RsrchBoy&url=https%3A%2F%2Fgithub.com%2FRsrchBoy%2FTest-Moose-More&title=RsrchBoy's%20CPAN%20Test-Moose-More&tags=%22RsrchBoy's%20Test-Moose-More%20in%20the%20CPAN%22"><img src="http://api.flattr.com/button/flattr-badge-large.png" /></a>
-
-=end html
-
-Please note B<I do not expect to be gittip'ed or flattr'ed for this work>,
-rather B<it is simply a very pleasant surprise>. I largely create and release
-works like this because I need them or I find it enjoyable; however, don't let
-that stop you if you feel like it ;)
-
-L<Flattr|https://flattr.com/submit/auto?user_id=RsrchBoy&url=https%3A%2F%2Fgithub.com%2FRsrchBoy%2FTest-Moose-More&title=RsrchBoy's%20CPAN%20Test-Moose-More&tags=%22RsrchBoy's%20Test-Moose-More%20in%20the%20CPAN%22>,
-L<Gratipay|https://gratipay.com/RsrchBoy/>, or indulge my
-L<Amazon Wishlist|http://bit.ly/rsrchboys-wishlist>...  If and *only* if you so desire.
-
 =head1 CONTRIBUTORS
 
 =for stopwords Chad Granum Karen Etheridge
